/*
 * statistics.h
 *
 *  Created on: 2010/10/18
 *      Author: shu
 */

#ifndef STATISTICS_H_
#define STATISTICS_H_

#include "score_matrix.h"
#include "encoder.h"
#include "sequence_type.h"
#include <vector>
#include <stdint.h>

using namespace std;

typedef struct KarlinParameters {
  float lambda;
  float K;
  float H;
} KarlinBlk;

class Statistics {
public:
  static float Nominal2Normalized(int normal_score, const KarlinParameters &paramters);
  static int Normalized2Nominal(float normalized_score, const KarlinParameters &paramters);
  static int NormalizedCutoff2NominalCutoff(float normalized_cutoff, const KarlinParameters &paramters);

  static double Nominal2EValue(int normal_score, uint64_t search_space, const KarlinParameters &paramters);

  Statistics(SequenceType &type);
  virtual ~Statistics();

  uint64_t CalculateSearchSpace(uint64_t query_length, uint64_t db_length, uint64_t db_number_sequences, const KarlinParameters &parameters, double alpha, double beta);
  uint64_t CalculateLengthAdjustment(uint64_t query_length, uint64_t db_length, uint64_t db_number_sequences, const KarlinParameters &parameters, double alpha, double beta);


  void CalculateUngappedIdealKarlinParameters(
      const ScoreMatrix &score_matrix, KarlinParameters *paramters) ;
  void CalculateUngappedKarlinParameters(const Code *query, uint32_t length,
      const ScoreMatrix &score_matrix, KarlinParameters *paramters) ;
  void CalculateGappedKarlinParameters(const ScoreMatrix &score_matrix,
      int open_gap, int extend_gap, KarlinParameters *parameters) ;
  void CalculateAlphaBeta(const ScoreMatrix &score_matrix, int open_gap, int extend_gap, double *alpha, double *beta) ;
private:
  void CalculateScoreProbabilities(const ScoreMatrix &score_matrix,
      double *score_frequency1, double *score_frequency2,
      double *score_probabilities);
  void Normalize(double *frequency, double norm);
  void ComposeSequenceFrequency(const Code *sequence, uint32_t length,
      double *frequency);

  SequenceTypePtr type_;
  vector<double> letter_prob_;
  Code min_regular_letter_code_;
  Code max_regular_letter_code_;
  Code min_code_;
  Code max_code_;

};

#endif /* STATISTICS_H_ */
